@@ -1,68 +1,39 @@
-#' Quosure getters, setters and testers
+#' Quosure getters, setters and predicates
 #'
 #' @description
+#' These tools inspect and modify [quosures][topic-quosure], a type of
+#' [defused expression][topic-defuse] that includes a reference to the
+#' context where it was created. A quosure is guaranteed to evaluate
+#' in its original environment and can refer to local objects safely.
 #'
-#' A quosure is a type of [quoted expression][quotation] that includes
-#' a reference to the context where it was created. A quosure is thus
-#' guaranteed to evaluate in its original environment and can refer to
-#' local objects.
+#' -  You can access the quosure components with `quo_get_expr()` and
+#'   `quo_get_env()`.
 #'
-#' You can access the quosure components (its expression and its
-#' environment) with:
+#' - The `quo_` prefixed predicates test the expression of a quosure,
+#'   `quo_is_missing()`, `quo_is_symbol()`, etc.
 #'
-#' * [get_expr()] and [get_env()]. These getters also support other
-#'   kinds of objects such as formulas.
-#'
-#' * `quo_get_expr()` and `quo_get_env()`. These getters only work
-#'   with quosures and throw an error with other types of input.
-#'
-#' Test if an object is a quosure with `is_quosure()`. If you know an
-#' object is a quosure, use the `quo_` prefixed predicates to check
-#' its contents, `quo_is_missing()`, `quo_is_symbol()`, etc.
-#'
-#'
-#' @section Quosured constants:
-#'
-#' A quosure usually does not carry environments for [constant
-#' objects][is_syntactic_literal] like strings or numbers. [quo()] and
-#' [enquo()] only capture an environment for [symbolic
-#' expressions][is_symbolic]. For instance, all of these return the
-#' [empty environment][empty_env]:
-#'
-#' ```
-#' quo_get_env(quo("constant"))
-#' quo_get_env(quo(100))
-#' quo_get_env(quo(NA))
-#' ```
-#'
-#' On the other hand, quosures capture the environment of symbolic
-#' expressions, i.e. expressions whose meaning depends on the
-#' environment in which they are evaluated and what objects are
-#' defined there:
-#'
-#' ```
-#' quo_get_env(quo(some_object))
-#' quo_get_env(quo(some_function()))
-#' ```
-#'
-#'
-#' @section Empty quosures:
+#' All `quo_` prefixed functions expect a quosure and will fail if
+#' supplied another type of object. Make sure the input is a quosure
+#' with [is_quosure()].
 #'
+#' @section Empty quosures and missing arguments:
 #' When missing arguments are captured as quosures, either through
 #' [enquo()] or [quos()], they are returned as an empty quosure. These
 #' quosures contain the [missing argument][missing_arg] and typically
 #' have the [empty environment][empty_env] as enclosure.
 #'
+#' Use `quo_is_missing()` to test for a missing argument defused with
+#' [enquo()].
 #'
-#' @section Life cycle:
+#' @seealso
+#' * [quo()] for creating quosures by [argument defusal][topic-defuse].
+#' * [new_quosure()] and [as_quosure()] for assembling quosures from
+#'   components.
+#' * `r link("topic_quosure")` for an overview.
 #'
-#' - `is_quosure()` is stable.
+#' @name quosure-tools
+#' @aliases quosure
 #'
-#' - `quo_get_expr()` and `quo_get_env()` are stable.
-#'
-#' @name quosure
-#' @seealso [quo()] for creating quosures by quotation; [as_quosure()]
-#'   and [new_quosure()] for constructing quosures manually.
 #' @examples
 #' quo <- quo(my_quosure)
 #' quo
@@ -95,66 +66,156 @@
 #' quo_is_missing(fn())
 NULL
 
-#' @rdname quosure
-#' @param x An object to test.
-#' @export
-is_quosure <- function(x) {
-  inherits(x, "quosure")
-}
-
-#' @rdname quosure
+#' @rdname quosure-tools
 #' @param quo A quosure to test.
 #' @export
 quo_is_missing <- function(quo) {
-  .Call(rlang_quo_is_missing, quo)
+  .Call(ffi_quo_is_missing, quo)
 }
-#' @rdname quosure
+#' @rdname quosure-tools
 #' @param name The name of the symbol or function call. If `NULL` the
 #'   name is not tested.
 #' @export
 quo_is_symbol <- function(quo, name = NULL) {
   is_symbol(quo_get_expr(quo), name = name)
 }
-#' @rdname quosure
+#' @rdname quosure-tools
 #' @inheritParams is_call
 #' @export
 quo_is_call <- function(quo, name = NULL, n = NULL, ns = NULL) {
   is_call(quo_get_expr(quo), name = name, n = n, ns = ns)
 }
-#' @rdname quosure
+#' @rdname quosure-tools
 #' @export
 quo_is_symbolic <- function(quo) {
-  .Call(rlang_quo_is_symbolic, quo)
+  .Call(ffi_quo_is_symbolic, quo)
 }
-#' @rdname quosure
+#' @rdname quosure-tools
 #' @export
 quo_is_null <- function(quo) {
-  .Call(rlang_quo_is_null, quo)
+  .Call(ffi_quo_is_null, quo)
 }
 
-
-#' @rdname quosure
+#' @rdname quosure-tools
 #' @export
 quo_get_expr <- function(quo) {
-  .Call(rlang_quo_get_expr, quo)
+  .Call(ffi_quo_get_expr, quo)
 }
-#' @rdname quosure
+#' @rdname quosure-tools
 #' @export
 quo_get_env <- function(quo) {
-  .Call(rlang_quo_get_env, quo)
+  .Call(ffi_quo_get_env, quo)
 }
 
-#' @rdname quosure
+#' @rdname quosure-tools
 #' @param expr A new expression for the quosure.
 #' @export
 quo_set_expr <- function(quo, expr) {
-  .Call(rlang_quo_set_expr, quo, expr)
+  .Call(ffi_quo_set_expr, quo, expr)
 }
-#' @rdname quosure
+#' @rdname quosure-tools
 #' @param env A new environment for the quosure.
 #' @export
 quo_set_env <- function(quo, env) {
-  .Call(rlang_quo_set_env, quo, env)
+  .Call(ffi_quo_set_env, quo, env)
+}
+
+
+#' Create a quosure from components
+#'
+#' @description
+#' * `new_quosure()` wraps any R object (including expressions,
+#'   formulas, or other quosures) into a [quosure][topic-quosure].
+#'
+#' * `as_quosure()` is similar but it does not rewrap formulas and
+#'   quosures.
+#'
+#' @param x For `is_quosure()`, an object to test. For `as_quosure()`,
+#'   an object to convert.
+#' @param expr An expression to wrap in a quosure.
+#' @param env The environment in which the expression should be
+#'   evaluated. Only used for symbols and calls. This should normally
+#'   be the environment in which the expression was created.
+#'
+#' @seealso
+#' * [enquo()] and [quo()] for creating a quosure by [argument
+#'   defusal][topic-defuse].
+#'
+#' * `r link("topic_quosure")`
+#' @examples
+#' # `new_quosure()` creates a quosure from its components. These are
+#' # equivalent:
+#' new_quosure(quote(foo), current_env())
+#'
+#' quo(foo)
+#'
+#' # `new_quosure()` always rewraps its input into a new quosure, even
+#' # if the input is itself a quosure:
+#' new_quosure(quo(foo))
+#'
+#' # This is unlike `as_quosure()` which preserves its input if it's
+#' # already a quosure:
+#' as_quosure(quo(foo))
+#'
+#'
+#' # `as_quosure()` uses the supplied environment with naked expressions:
+#' env <- env(var = "thing")
+#' as_quosure(quote(var), env)
+#'
+#' # If the expression already carries an environment, this
+#' # environment is preserved. This is the case for formulas and
+#' # quosures:
+#' as_quosure(~foo, env)
+#'
+#' as_quosure(~foo)
+#'
+#' # An environment must be supplied when the input is a naked
+#' # expression:
+#' try(
+#'   as_quosure(quote(var))
+#' )
+#' @export
+new_quosure <- function(expr, env = caller_env()) {
+  .Call(ffi_new_quosure, expr, env)
+}
+#' @rdname new_quosure
+#' @export
+as_quosure <- function(x, env = NULL) {
+  if (is_quosure(x)) {
+    return(x)
+  }
+
+  if (is_bare_formula(x)) {
+    env <- f_env(x)
+    if (is_null(env)) {
+      abort(paste_line(
+        "The formula does not have an environment.",
+        "This is a quoted formula that was never evaluated."
+      ))
+    }
+
+    return(new_quosure(f_rhs(x), env))
+  }
+
+  if (is_symbolic(x)) {
+    if (is_null(env)) {
+      warn_deprecated(paste_line(
+        "`as_quosure()` requires an explicit environment as of rlang 0.3.0.",
+        "Please supply `env`."
+      ))
+      env <- caller_env()
+    }
+
+    return(new_quosure(x, env))
+  }
+
+  new_quosure(x, empty_env())
+}
+#' @rdname new_quosure
+#' @param x An object to test.
+#' @export
+is_quosure <- function(x) {
+  inherits(x, "quosure")
 }
 
 
@@ -182,7 +243,7 @@ new_quosures <- function(x) {
     abort("Expected a list of quosures")
   }
   structure(x,
-    class = "quosures",
+    class = c("quosures", "list"),
     names = names2(x)
   )
 }
@@ -203,7 +264,7 @@ is_quosures <- function(x) {
 
 #' @export
 `[.quosures` <- function(x, i) {
-  structure(NextMethod(), class = "quosures")
+  structure(NextMethod(), class = c("quosures", "list"))
 }
 #' @export
 c.quosures <- function(..., recursive = FALSE) {
@@ -254,90 +315,17 @@ signal_quosure_assign <- function(x) {
   ))
 }
 
-# Dynamically registered
 pillar_shaft.quosures <- function(x, ...) {
   labels <- map_chr(unname(x), as_label)
   structure(labels, width = 10L)
 }
-type_sum.quosures <- function(x) {
+type_sum.quosures <- function(x, ...) {
   "quos"
 }
-
-#' Coerce object to quosure
-#'
-#' @description
-#'
-#' While `new_quosure()` wraps any R object (including expressions,
-#' formulas, or other quosures) into a quosure, `as_quosure()`
-#' converts formulas and quosures and does not double-wrap.
-#'
-#'
-#' @section Life cycle:
-#'
-#' - `as_quosure()` now requires an explicit default environment for
-#'   creating quosures from symbols and calls.
-#'
-#' - `as_quosureish()` is deprecated as of rlang 0.2.0. This function
-#'   assumes that quosures are formulas which is currently true but
-#'   might not be in the future.
-#'
-#' @param x An object to convert. Either an [expression][is_expression] or a
-#'   formula.
-#' @param env The environment in which the expression should be
-#'   evaluated. Only used for symbols and calls. This should typically
-#'   be the environment in which the expression was created.
-#' @seealso [quo()], [is_quosure()]
-#' @export
-#' @examples
-#' # as_quosure() converts expressions or any R object to a validly
-#' # scoped quosure:
-#' env <- env(var = "thing")
-#' as_quosure(quote(var), env)
-#'
-#'
-#' # The environment is ignored for formulas:
-#' as_quosure(~foo, env)
-#' as_quosure(~foo)
-#'
-#' # However you must supply it for symbols and calls:
-#' try(as_quosure(quote(var)))
-as_quosure <- function(x, env = NULL) {
-  if (is_quosure(x)) {
-    return(x)
-  }
-
-  if (is_bare_formula(x)) {
-    env <- f_env(x)
-    if (is_null(env)) {
-      abort(paste_line(
-        "The formula does not have an environment.",
-        "This is a quoted formula that was never evaluated."
-      ))
-    }
-
-    return(new_quosure(f_rhs(x), env))
-  }
-
-  if (is_symbolic(x)) {
-    if (is_null(env)) {
-      warn_deprecated(paste_line(
-        "`as_quosure()` requires an explicit environment as of rlang 0.3.0.",
-        "Please supply `env`."
-      ))
-      env <- caller_env()
-    }
-
-    return(new_quosure(x, env))
-  }
-
-  new_quosure(x, empty_env())
-}
-#' @rdname as_quosure
-#' @param expr The expression wrapped by the quosure.
-#' @export
-new_quosure <- function(expr, env = caller_env()) {
-  .Call(rlang_new_quosure, expr, env)
-}
+on_load({
+  s3_register("pillar::pillar_shaft", "quosures", pillar_shaft.quosures)
+  s3_register("pillar::type_sum", "quosures", type_sum.quosures)
+})
 
 
 #' Squash a quosure
@@ -349,20 +337,12 @@ new_quosure <- function(expr, env = caller_env()) {
 #' expression `foo(bar(), baz)`.
 #'
 #' This operation is safe if the squashed quosure is used for
-#' labelling or printing (see [quo_label()] or [quo_name()]). However
-#' if the squashed quosure is evaluated, all expressions of the
-#' flattened quosures are resolved in a single environment. This is a
-#' source of bugs so it is good practice to set `warn` to `TRUE` to
-#' let the user know about the lossy squashing.
-#'
-#'
-#' @section Life cycle:
-#'
-#' This function replaces `quo_expr()` which was deprecated in
-#' rlang 0.2.0. `quo_expr()` was a misnomer because it implied that it
-#' was a mere expression acccessor for quosures whereas it was really
-#' a lossy operation that squashed all nested quosures.
-#'
+#' labelling or printing (see [as_label()], but note that `as_label()`
+#' squashes quosures automatically). However if the squashed quosure
+#' is evaluated, all expressions of the flattened quosures are
+#' resolved in a single environment. This is a source of bugs so it is
+#' good practice to set `warn` to `TRUE` to let the user know about
+#' the lossy squashing.
 #'
 #' @param quo A quosure or expression.
 #' @param warn Whether to warn if the quosure contains other quosures
@@ -395,9 +375,10 @@ quo_squash <- function(quo, warn = FALSE) {
 
 #' Format quosures for printing or labelling
 #'
+#' @keywords internal
 #' @description
 #'
-#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("questioning")}
+#' `r lifecycle::badge("questioning")`
 #'
 #' **Note:** You should now use [as_label()] or [as_name()] instead
 #' of `quo_name()`. See life cycle section below.
@@ -472,7 +453,8 @@ quo_name <- function(quo) {
 }
 
 quo_squash_impl <- function(x, parent = NULL, warn = FALSE) {
-  switch_expr(x,
+  switch_expr(
+    x,
     language = {
       if (is_quosure(x)) {
         if (!is_false(warn)) {
@@ -485,11 +467,11 @@ quo_squash_impl <- function(x, parent = NULL, warn = FALSE) {
           warn <- FALSE
         }
 
-        while (is_quosure(x)) {
+        while (is_quosure(maybe_missing(x))) {
           x <- quo_get_expr(x)
         }
         if (!is_null(parent)) {
-          node_poke_car(parent, x)
+          node_poke_car(parent, maybe_missing(x))
         }
         quo_squash_impl(x, parent, warn = warn)
       } else {
@@ -504,7 +486,7 @@ quo_squash_impl <- function(x, parent = NULL, warn = FALSE) {
     }
   )
 
-  x
+  maybe_missing(x)
 }
 
 
@@ -580,7 +562,7 @@ quo_deparse <- function(x, lines = new_quo_deparser()) {
 
   lines$push("^")
   lines$make_next_sticky()
-  sexp_deparse(quo_get_expr(x), lines)
+  sexp_deparse(quo_get_expr(x), lines = lines)
 
   lines$quo_reset_colour()
 
@@ -588,8 +570,13 @@ quo_deparse <- function(x, lines = new_quo_deparser()) {
 }
 
 new_quo_deparser <- function(width = peek_option("width"),
+                             max_elements = 5L,
                              crayon = has_crayon()) {
-  lines <- new_lines(width = width, deparser = quo_deparse)
+  lines <- new_lines(
+    width = width,
+    max_elements = max_elements,
+    deparser = quo_deparse
+  )
 
   child_r6lite(lines,
     has_colour = crayon,
