@@ -11,21 +11,17 @@ extern r_obj* rlang_linked_version();
 
 static const R_CallMethodDef r_callables[] = {
   {"ffi_alloc_data_frame",             (DL_FUNC) &ffi_alloc_data_frame, 3},
-  {"ffi_arr_info",                     (DL_FUNC) &ffi_arr_info, 1},
-  {"ffi_arr_pop_back",                 (DL_FUNC) &ffi_arr_pop_back, 1},
-  {"ffi_arr_push_back",                (DL_FUNC) &ffi_arr_push_back, 2},
-  {"ffi_arr_push_back_bool",           (DL_FUNC) &ffi_arr_push_back_bool, 2},
-  {"ffi_arr_resize",                   (DL_FUNC) &ffi_arr_resize, 2},
-  {"ffi_arr_unwrap",                   (DL_FUNC) &ffi_arr_unwrap, 1},
   {"ffi_as_data_mask",                 (DL_FUNC) &ffi_as_data_mask, 1},
   {"ffi_as_data_pronoun",              (DL_FUNC) &ffi_as_data_pronoun, 1},
   {"ffi_attrib",                       (DL_FUNC) &r_attrib, 1},
   {"ffi_c_tests",                      (DL_FUNC) &ffi_c_tests, 0},
   {"ffi_call_has_precedence",          (DL_FUNC) &ffi_call_has_precedence, 3},
+  {"ffi_call_zap_inline",              (DL_FUNC) &ffi_call_zap_inline, 1},
   {"ffi_chr_get",                      (DL_FUNC) &ffi_chr_get, 2},
   {"ffi_chr_has_curly",                (DL_FUNC) &ffi_chr_has_curly, 1},
   {"ffi_cnd_signal",                   (DL_FUNC) &ffi_cnd_signal, 1},
   {"ffi_cnd_type",                     (DL_FUNC) &ffi_cnd_type, 1},
+  {"ffi_compiled_by_gcc",              (DL_FUNC) &ffi_compiled_by_gcc, 0},
   {"ffi_data_mask_clean",              (DL_FUNC) &ffi_data_mask_clean, 1},
   {"ffi_data_pronoun_get",             (DL_FUNC) &ffi_data_pronoun_get, 3},
   {"ffi_dict_as_df_list",              (DL_FUNC) &ffi_dict_as_df_list, 1},
@@ -42,6 +38,26 @@ static const R_CallMethodDef r_callables[] = {
   {"ffi_dots_list",                    (DL_FUNC) &ffi_dots_list, 7},
   {"ffi_dots_pairlist",                (DL_FUNC) &ffi_dots_pairlist, 7},
   {"ffi_duplicate",                    (DL_FUNC) &ffi_duplicate, 2},
+  {"ffi_dyn_chr_get",                  (DL_FUNC) &ffi_dyn_chr_get, 2},
+  {"ffi_dyn_chr_poke",                 (DL_FUNC) &ffi_dyn_chr_poke, 3},
+  {"ffi_dyn_cpl_get",                  (DL_FUNC) &ffi_dyn_cpl_get, 2},
+  {"ffi_dyn_cpl_poke",                 (DL_FUNC) &ffi_dyn_cpl_poke, 3},
+  {"ffi_dyn_dbl_get",                  (DL_FUNC) &ffi_dyn_dbl_get, 2},
+  {"ffi_dyn_dbl_poke",                 (DL_FUNC) &ffi_dyn_dbl_poke, 3},
+  {"ffi_dyn_info",                     (DL_FUNC) &ffi_dyn_info, 1},
+  {"ffi_dyn_int_get",                  (DL_FUNC) &ffi_dyn_int_get, 2},
+  {"ffi_dyn_int_poke",                 (DL_FUNC) &ffi_dyn_int_poke, 3},
+  {"ffi_dyn_lgl_get",                  (DL_FUNC) &ffi_dyn_lgl_get, 2},
+  {"ffi_dyn_lgl_poke",                 (DL_FUNC) &ffi_dyn_lgl_poke, 3},
+  {"ffi_dyn_list_get",                 (DL_FUNC) &ffi_dyn_list_get, 2},
+  {"ffi_dyn_list_poke",                 (DL_FUNC) &ffi_dyn_list_poke, 3},
+  {"ffi_dyn_pop_back",                 (DL_FUNC) &ffi_dyn_pop_back, 1},
+  {"ffi_dyn_push_back",                (DL_FUNC) &ffi_dyn_push_back, 2},
+  {"ffi_dyn_push_back_bool",           (DL_FUNC) &ffi_dyn_push_back_bool, 2},
+  {"ffi_dyn_raw_get",                  (DL_FUNC) &ffi_dyn_raw_get, 2},
+  {"ffi_dyn_raw_poke",                 (DL_FUNC) &ffi_dyn_raw_poke, 3},
+  {"ffi_dyn_resize",                   (DL_FUNC) &ffi_dyn_resize, 2},
+  {"ffi_dyn_unwrap",                   (DL_FUNC) &ffi_dyn_unwrap, 1},
   {"ffi_ellipsis_dots",                (DL_FUNC) &ffi_ellipsis_dots, 1},
   {"ffi_ellipsis_dots_used",           (DL_FUNC) &ffi_ellipsis_dots_used, 1},
   {"ffi_ellipsis_promise_forced",      (DL_FUNC) &ffi_ellipsis_promise_forced, 1},
@@ -53,6 +69,7 @@ static const R_CallMethodDef r_callables[] = {
   {"ffi_env_binding_types",            (DL_FUNC) &r_env_binding_types, 2},
   {"ffi_env_browse",                   (DL_FUNC) &ffi_env_browse, 2},
   {"ffi_env_clone",                    (DL_FUNC) &r_env_clone, 2},
+  {"ffi_env_coalesce",                 (DL_FUNC) &ffi_env_coalesce, 2},
   {"ffi_env_frame",                    (DL_FUNC) &ffi_env_frame, 1},
   {"ffi_env_get",                      (DL_FUNC) &ffi_env_get, 5},
   {"ffi_env_get_list",                 (DL_FUNC) &ffi_env_get_list, 5},
@@ -73,6 +90,7 @@ static const R_CallMethodDef r_callables[] = {
   {"ffi_fini_rlang",                   (DL_FUNC) &ffi_fini_rlang, 0},
   {"ffi_format_error_arg",             (DL_FUNC) &ffi_format_error_arg, 1},
   {"ffi_get_expression",               (DL_FUNC) &ffi_get_expression, 2},
+  {"ffi_getppid",                      (DL_FUNC) &ffi_getppid, 0},
   {"ffi_glue_is_here",                 (DL_FUNC) &ffi_glue_is_here, 0},
   {"ffi_has_local_precious_list",      (DL_FUNC) &ffi_has_local_precious_list, 0},
   {"ffi_hash",                         (DL_FUNC) &ffi_hash, 1},
@@ -85,6 +103,7 @@ static const R_CallMethodDef r_callables[] = {
   {"ffi_interp",                       (DL_FUNC) &ffi_interp, 2},
   {"ffi_interrupt",                    (DL_FUNC) &ffi_interrupt, 0},
   {"ffi_is_atomic",                    (DL_FUNC) &ffi_is_atomic, 2},
+  {"ffi_is_call",                      (DL_FUNC) &ffi_is_call, 4},
   {"ffi_is_character",                 (DL_FUNC) &ffi_is_character, 4},
   {"ffi_is_closure",                   (DL_FUNC) &ffi_is_closure, 1},
   {"ffi_is_complex",                   (DL_FUNC) &ffi_is_complex, 3},
@@ -195,6 +214,7 @@ static const R_CallMethodDef r_callables[] = {
   {"ffi_test_parse_eval",              (DL_FUNC) &ffi_test_parse_eval, 2},
   {"ffi_test_r_on_exit",               (DL_FUNC) &r_on_exit, 2},
   {"ffi_test_r_warn",                  (DL_FUNC) &ffi_test_r_warn, 1},
+  {"ffi_test_stop_internal",           (DL_FUNC) &ffi_test_stop_internal, 1},
   {"ffi_test_sys_call",                (DL_FUNC) &ffi_test_sys_call, 1},
   {"ffi_test_sys_frame",               (DL_FUNC) &ffi_test_sys_frame, 1},
   {"ffi_true_length",                  (DL_FUNC) &ffi_true_length, 1},
@@ -216,7 +236,7 @@ static const R_CallMethodDef r_callables[] = {
 
 
 static const R_ExternalMethodDef externals[] = {
-  {"ffi_arg_match0",                    (DL_FUNC) &ffi_arg_match0, 4},
+  {"ffi_arg_match0",                    (DL_FUNC) &ffi_arg_match0, 3},
   {"ffi_call2",                         (DL_FUNC) &ffi_call2, 2},
   {"ffi_capturearginfo",                (DL_FUNC) &ffi_capturearginfo, 2},
   {"ffi_capturedots",                   (DL_FUNC) &ffi_capturedots, 1},
@@ -242,7 +262,8 @@ extern uint64_t XXH3_64bits(const void*, size_t);
 
 r_visible
 void R_init_rlang(DllInfo* dll) {
-  R_RegisterCCallable("rlang", "rlang_arg_match",           (DL_FUNC) &arg_match);
+  R_RegisterCCallable("rlang", "rlang_arg_match",           (DL_FUNC) &arg_match_legacy);
+  R_RegisterCCallable("rlang", "rlang_arg_match_2",         (DL_FUNC) &cci_arg_match);
   R_RegisterCCallable("rlang", "rlang_as_data_mask_3.0.0",  (DL_FUNC) &ffi_as_data_mask);
   R_RegisterCCallable("rlang", "rlang_as_data_pronoun",     (DL_FUNC) &ffi_as_data_pronoun);
   R_RegisterCCallable("rlang", "rlang_env_unbind",          (DL_FUNC) &r_env_unbind);
@@ -257,6 +278,7 @@ void R_init_rlang(DllInfo* dll) {
   R_RegisterCCallable("rlang", "rlang_quo_set_env",         (DL_FUNC) &ffi_quo_set_env);
   R_RegisterCCallable("rlang", "rlang_quo_set_expr",        (DL_FUNC) &ffi_quo_set_expr);
   R_RegisterCCallable("rlang", "rlang_stop_internal",       (DL_FUNC) &rlang_stop_internal);
+  R_RegisterCCallable("rlang", "rlang_stop_internal2",      (DL_FUNC) &rlang_stop_internal2);
 
   r_obj* r_as_function(r_obj* x, const char* arg);
   R_RegisterCCallable("rlang", "rlang_as_function",         (DL_FUNC) &r_as_function);
@@ -282,6 +304,7 @@ void R_init_rlang(DllInfo* dll) {
 
   // Only for debugging - no stability guaranteed
   R_RegisterCCallable("rlang", "rlang_print_backtrace",     (DL_FUNC) &rlang_print_backtrace);
+  R_RegisterCCallable("rlang", "rlang_env_print",           (DL_FUNC) &rlang_env_print);
 
   R_registerRoutines(dll, NULL, r_callables, NULL, externals);
   R_useDynamicSymbols(dll, FALSE);
