@@ -169,6 +169,7 @@ test_that("entrace() preserves exit status in non-interactive sessions (#1052, r
   # Probably because of <https://github.com/wch/r-source/commit/3055aa86>
   skip_if(getRversion() < "3.3")
 
+  # This also tests for empty backtraces
   out <- Rscript(shQuote(c("--vanilla", "-e", 'options(error = rlang::entrace); stop("An error")')))
   expect_false(out$status == 0L)
 
@@ -228,7 +229,7 @@ test_that("can supply handler environment as `bottom`", {
 })
 
 test_that("can set `entrace()` as a global handler", {
-  skip_if_not_installed("base", "4.0")
+  skip_if_not_installed("base", "4.0.0")
 
   expect_snapshot_output(run('{
     suppressMessages(testthat::local_reproducible_output())
