@@ -65,18 +65,6 @@ test_that("flatten_if() handles custom predicate", {
   expect_identical(flatten_if(x, pred), list(obj, obj[[1]], obj[[1]]))
 })
 
-test_that("flatten_if() handles external pointers", {
-  obj <- structure(list(1:2), class = "foo")
-  x <- list(obj, splice(obj), unclass(obj))
-
-  expect_identical(flatten_if(x, rlang_test_is_spliceable), list(obj[[1]], splice(obj), unclass(obj)))
-
-  ptr <- rlang_test_is_spliceable[[1]]
-  expect_identical(flatten_if(x, ptr), list(obj[[1]], splice(obj), unclass(obj)))
-
-  expect_s3_class(rlang_test_is_spliceable, "fn_pointer")
-})
-
 test_that("flatten() splices names", {
   expect_warning(regexp = "Outer names",
     expect_identical(
